package es.uca.xmleye;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA

 */

import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;
import javax.swing.*;
import java.util.*;
import com.jgoodies.looks.plastic.*;
import com.jgoodies.looks.plastic.theme.*;

import es.uca.xmleye.gui.*;
import es.uca.xmleye.prefs.*;
import es.uca.xmleye.types.TypeDescriptorException;
import es.uca.xmleye.xslt.TransformationException;

public class Application {
	/** Versión del visor. */
	public final static String VERSION = "1.21";

	// Referencia para comunicación con la UI
	private final WndMain wndPrincipal;

	// Internacionalización de mensajes de error
	private final static ResourceBundle errRes;

	static {
		errRes = ResourceBundle.getBundle("es.uca.xmleye.l10n.ErrorMessages",
				Locale.getDefault());
	}

	/**
	 * <p>
	 * Inicializa la aplicacion.
	 * </p>
	 * 
	 * <ol>
	 * <li>Carga el L&amp;F personalizado (el L&amp;F por defecto es poco
	 * estético).</li>
	 * <li>Crea los subcontroladores.</li>
	 * <li>Muestra la ventana principal.</li>
	 * </ol>
	 * 
	 * @param docsPorAbrir
	 *            Documentos que se desean abrir pasados desde la línea de
	 *            órdenes.
	 */
	public Application(String[] docsPorAbrir) throws ParsingException,
			TransformationException, TypeDescriptorException {
		// No existe otra forma sencilla de activar antialiasing en fuentes.
		// Ésta sólo funciona en J2SE 1.5, y no está documentada en el API
		// oficial.
		System.setProperty("swing.aatext", "true");

		// Usa el look and feel de JGoodies si puedes: aseguramos el
		// mismo aspecto entre versiones y sistemas operativos.
		Plastic3DLookAndFeel.setPlasticTheme(new ExperienceBlue());
		try {
			UIManager.setLookAndFeel(new Plastic3DLookAndFeel());
		} catch (Exception e) {
			System.err.println(errRes.getString("err_lookandfeel_notfound"));
		}

		// Cargamos las preferencias
		try {
			PreferencesManagerSingleton.getInstancia().cargar();
		} catch (PreferencesException pEx) {
			MessageManagerSingleton.getInstancia().mostrarError(
					errRes.getString("err_loadpref_pex") + "\n"
							+ pEx.getLocalizedMessage());
		}

		// Procesamos los ficheros pasados para ver si son URL o no
		ArrayList<File> fichPorAbrir = new ArrayList<File>();
		for (String ruta : docsPorAbrir) {
			try {
				if (ruta.matches("[a-z]+://.*")) {
					fichPorAbrir.add(new File(new URI(ruta)));
				} else {
					fichPorAbrir.add(new File(ruta));
				}
			} catch (URISyntaxException ex) {
			}
		}

		// Iniciamos el interfaz gráfico
		wndPrincipal = new WndMain("XMLEye", this, fichPorAbrir);
		MessageManagerSingleton.getInstancia().setCompPadre(wndPrincipal);
	}

	/**
	 * Cierra el programa tras realizar todas las operaciones necesarias.
	 */
	public void cerrar() {
		try {
			PreferencesManagerSingleton.getInstancia().guardar();
		} catch (PreferencesException prefEx) {
			MessageManagerSingleton.getInstancia().mostrarError(
					String.format(errRes.getString("err_savepref_pex")) + "\n"
							+ prefEx.getLocalizedMessage());
		}

		System.exit(0);
	}

	/**
	 * Punto de inicio de la aplicación.
	 */
	public static void main(String[] args) throws ParsingException,
			TransformationException, TypeDescriptorException {
		new Application(args);
	}

}
